@@ -95,6 +95,16 @@ by supplying ``--sql-mode`` to ``mysqld``.  You can also use a ``Pool`` hook
 to issue a ``SET SESSION sql_mode='...'`` on connect to configure each
 connection.
 
+If you do not specify 'use_ansiquotes', the regular MySQL quoting style is
+used by default.  Table reflection operations will query the server 
+
+If you do issue a 'SET sql_mode' through SQLAlchemy, the dialect must be
+updated if the quoting style is changed.  Again, this change will affect all
+connections::
+
+  connection.execute('SET sql_mode="ansi"')
+  connection.dialect.use_ansiquotes = True
+
 For normal SQLAlchemy usage, loading this module is unnescesary.  It will be
 loaded on-demand when a MySQL connection is needed.  The generic column types
 like ``String`` and ``Integer`` will automatically be adapted to the optimal
@@ -143,11 +153,12 @@ Notes page on the wiki at http://www.sqlalchemy.org is a good resource for
 timely information affecting MySQL in SQLAlchemy.
 """
 
-import re, datetime, inspect, warnings, sys
+import datetime, inspect, re, sys
 from array import array as _array
 
 from sqlalchemy import exceptions, logging, schema, sql, util
 from sqlalchemy.sql import operators as sql_operators
+from sqlalchemy.sql import functions as sql_functions
 from sqlalchemy.sql import compiler
 
 from sqlalchemy.engine import base as engine_base, default
@@ -212,6 +223,10 @@ AUTOCOMMIT_RE = re.compile(
 SELECT_RE = re.compile(
     r'\s*(?:SELECT|SHOW|DESCRIBE|XA RECOVER)',
     re.I | re.UNICODE)
+SET_RE = re.compile(
+    r'\s*SET\s+(?:(?:GLOBAL|SESSION)\s+)?\w',
+    re.I | re.UNICODE)
+
 
 class _NumericType(object):
     """Base for MySQL numeric types."""
@@ -1384,6 +1399,11 @@ class MySQLExecutionContext(default.DefaultExecutionContext):
                 self._last_inserted_ids[0] is None):
                 self._last_inserted_ids = ([self.cursor.lastrowid] +
                                            self._last_inserted_ids[1:])
+        elif (not self.isupdate and not self.should_autocommit and
+              self.statement and SET_RE.match(self.statement)):
+            # This misses if a user forces autocommit on text('SET NAMES'),
+            # which is probably a programming error anyhow.
+            self.connection.info.pop(('mysql', 'charset'), None)
 
     def returns_rows_text(self, statement):
         return SELECT_RE.match(statement)
@@ -1400,14 +1420,10 @@ class MySQLDialect(default.DefaultDialect):
     # identifiers are 64, however aliases can be 255...
     max_identifier_length = 255
     supports_sane_rowcount = True
+    default_paramstyle = 'format'
 
-    def __init__(self, use_ansiquotes=False, **kwargs):
+    def __init__(self, use_ansiquotes=None, **kwargs):
         self.use_ansiquotes = use_ansiquotes
-        kwargs.setdefault('default_paramstyle', 'format')
-        if self.use_ansiquotes:
-            self.preparer = MySQLANSIIdentifierPreparer
-        else:
-            self.preparer = MySQLIdentifierPreparer
         default.DefaultDialect.__init__(self, **kwargs)
 
     def dbapi(cls):
@@ -1535,17 +1551,15 @@ class MySQLDialect(default.DefaultDialect):
             return False
 
     def get_default_schema_name(self, connection):
-        try:
-            return self._default_schema_name
-        except AttributeError:
-            name = self._default_schema_name = \
-              connection.execute('SELECT DATABASE()').scalar()
-            return name
+        return connection.execute('SELECT DATABASE()').scalar()
+    get_default_schema_name = engine_base.connection_memoize(
+        ('dialect', 'default_schema_name'))(get_default_schema_name)
 
     def table_names(self, connection, schema):
         """Return a Unicode SHOW TABLES from a given schema."""
 
         charset = self._detect_charset(connection)
+        self._autoset_identifier_style(connection)
         rp = connection.execute("SHOW TABLES FROM %s" %
             self.identifier_preparer.quote_identifier(schema))
         return [row[0] for row in _compat_fetchall(rp, charset=charset)]
@@ -1558,7 +1572,10 @@ class MySQLDialect(default.DefaultDialect):
         # based on platform.  DESCRIBE is slower.
 
         # [ticket:726]
-        # full_name = self.identifier_preparer.format_table(table, use_schema=True)
+        # full_name = self.identifier_preparer.format_table(table,
+        #                                                   use_schema=True)
+
+        self._autoset_identifier_style(connection)
 
         full_name = '.'.join(self.identifier_preparer._quote_free_identifiers(
             schema, table_name))
@@ -1591,12 +1608,9 @@ class MySQLDialect(default.DefaultDialect):
         cached per-Connection.
         """
 
-        try:
-            return connection.info['_mysql_server_version_info']
-        except KeyError:
-            version = connection.info['_mysql_server_version_info'] = \
-              self._server_version_info(connection.connection.connection)
-            return version
+        return self._server_version_info(connection.connection.connection)
+    server_version_info = engine_base.connection_memoize(
+        ('mysql', 'server_version_info'))(server_version_info)
 
     def _server_version_info(self, dbapi_con):
         """Convert a MySQL-python server_info string into a tuple."""
@@ -1621,18 +1635,24 @@ class MySQLDialect(default.DefaultDialect):
             connectable = connectable.contextual_connect()
 
         return self.server_version_info(connectable)
-    get_version_info = util.deprecated(get_version_info)
+    get_version_info = util.deprecated()(get_version_info)
 
     def reflecttable(self, connection, table, include_columns):
         """Load column definitions from the server."""
 
         charset = self._detect_charset(connection)
+        self._autoset_identifier_style(connection)
 
         try:
             reflector = self.reflector
         except AttributeError:
-            self.reflector = reflector = \
-                MySQLSchemaReflector(self.identifier_preparer)
+            preparer = self.identifier_preparer
+            if (self.server_version_info(connection) < (4, 1) and
+                self.use_ansiquotes):
+                # ANSI_QUOTES doesn't affect SHOW CREATE TABLE on < 4.1
+                preparer = MySQLIdentifierPreparer(self)
+
+            self.reflector = reflector = MySQLSchemaReflector(preparer)
 
         sql = self._show_create_table(connection, table, charset)
         if sql.startswith('CREATE ALGORITHM'):
@@ -1640,7 +1660,7 @@ class MySQLDialect(default.DefaultDialect):
             columns = self._describe_table(connection, table, charset)
             sql = reflector._describe_to_create(table, columns)
 
-        self._adjust_casing(connection, table, charset)
+        self._adjust_casing(connection, table)
 
         return reflector.reflect(connection, table, sql, charset,
                                  only=include_columns)
@@ -1648,10 +1668,7 @@ class MySQLDialect(default.DefaultDialect):
     def _adjust_casing(self, connection, table, charset=None):
         """Adjust Table name to the server case sensitivity, if needed."""
 
-        if charset is None:
-            charset = self._detect_charset(connection)
-
-        casing = self._detect_casing(connection, charset)
+        casing = self._detect_casing(connection)
 
         # For winxx database hosts.  TODO: is this really needed?
         if casing == 1 and table.name != table.name.lower():
@@ -1664,8 +1681,8 @@ class MySQLDialect(default.DefaultDialect):
         """Sniff out the character set in use for connection results."""
 
         # Allow user override, won't sniff if force_charset is set.
-        if 'force_charset' in connection.info:
-            return connection.info['force_charset']
+        if ('mysql', 'force_charset') in connection.info:
+            return connection.info[('mysql', 'force_charset')]
 
         # Note: MySQL-python 1.2.1c7 seems to ignore changes made
         # on a connection via set_character_set()
@@ -1695,60 +1712,104 @@ class MySQLDialect(default.DefaultDialect):
             if 'character_set' in opts:
                 return opts['character_set']
             else:
-                warnings.warn(RuntimeWarning(
+                util.warn(
                     "Could not detect the connection character set with this "
                     "combination of MySQL server and MySQL-python. "
-                    "MySQL-python >= 1.2.2 is recommended.  Assuming latin1."))
+                    "MySQL-python >= 1.2.2 is recommended.  Assuming latin1.")
                 return 'latin1'
+    _detect_charset = engine_base.connection_memoize(
+        ('mysql', 'charset'))(_detect_charset)
 
-    def _detect_casing(self, connection, charset=None):
+
+    def _detect_casing(self, connection):
         """Sniff out identifier case sensitivity.
 
         Cached per-connection. This value can not change without a server
         restart.
-        """
 
+        """
         # http://dev.mysql.com/doc/refman/5.0/en/name-case-sensitivity.html
 
-        try:
-            return connection.info['lower_case_table_names']
-        except KeyError:
-            row = _compat_fetchone(connection.execute(
-                    "SHOW VARIABLES LIKE 'lower_case_table_names'"),
-                                   charset=charset)
-            if not row:
+        charset = self._detect_charset(connection)
+        row = _compat_fetchone(connection.execute(
+            "SHOW VARIABLES LIKE 'lower_case_table_names'"),
+                               charset=charset)
+        if not row:
+            cs = 0
+        else:
+            # 4.0.15 returns OFF or ON according to [ticket:489]
+            # 3.23 doesn't, 4.0.27 doesn't..
+            if row[1] == 'OFF':
                 cs = 0
+            elif row[1] == 'ON':
+                cs = 1
             else:
-                # 4.0.15 returns OFF or ON according to [ticket:489]
-                # 3.23 doesn't, 4.0.27 doesn't..
-                if row[1] == 'OFF':
-                    cs = 0
-                elif row[1] == 'ON':
-                    cs = 1
-                else:
-                    cs = int(row[1])
-                row.close()
-            connection.info['lower_case_table_names'] = cs
-            return cs
+                cs = int(row[1])
+            row.close()
+        return cs
+    _detect_casing = engine_base.connection_memoize(
+        ('mysql', 'lower_case_table_names'))(_detect_casing)
 
-    def _detect_collations(self, connection, charset=None):
+    def _detect_collations(self, connection):
         """Pull the active COLLATIONS list from the server.
 
         Cached per-connection.
         """
 
-        try:
-            return connection.info['collations']
-        except KeyError:
-            collations = {}
-            if self.server_version_info(connection) < (4, 1, 0):
-                pass
-            else:
-                rs = connection.execute('SHOW COLLATION')
-                for row in _compat_fetchall(rs, charset):
-                    collations[row[0]] = row[1]
-            connection.info['collations'] = collations
-            return collations
+        collations = {}
+        if self.server_version_info(connection) < (4, 1, 0):
+            pass
+        else:
+            charset = self._detect_charset(connection)
+            rs = connection.execute('SHOW COLLATION')
+            for row in _compat_fetchall(rs, charset):
+                collations[row[0]] = row[1]
+        return collations
+    _detect_collations = engine_base.connection_memoize(
+        ('mysql', 'collations'))(_detect_collations)
+
+    def use_ansiquotes(self, useansi):
+        self._use_ansiquotes = useansi
+        if useansi:
+            self.preparer = MySQLANSIIdentifierPreparer
+        else:
+            self.preparer = MySQLIdentifierPreparer
+        # icky
+        if hasattr(self, 'identifier_preparer'):
+            self.identifier_preparer = self.preparer(self)
+        if hasattr(self, 'reflector'):
+            del self.reflector
+
+    use_ansiquotes = property(lambda s: s._use_ansiquotes, use_ansiquotes,
+                              doc="True if ANSI_QUOTES is in effect.")
+
+    def _autoset_identifier_style(self, connection, charset=None):
+        """Detect and adjust for the ANSI_QUOTES sql mode.
+
+        If the dialect's use_ansiquotes is unset, query the server's sql mode
+        and reset the identifier style.
+
+        Note that this currently *only* runs during reflection.  Ideally this
+        would run the first time a connection pool connects to the database,
+        but the infrastructure for that is not yet in place.
+        """
+
+        if self.use_ansiquotes is not None:
+            return
+
+        row = _compat_fetchone(
+            connection.execute("SHOW VARIABLES LIKE 'sql_mode'",
+                               charset=charset))
+        if not row:
+            mode = ''
+        else:
+            mode = row[1] or ''
+            # 4.0
+            if mode.isdigit():
+                mode_no = int(mode)
+                mode = (mode_no | 4 == mode_no) and 'ANSI_QUOTES' or ''
+
+        self.use_ansiquotes = 'ANSI_QUOTES' in mode
 
     def _show_create_table(self, connection, table, charset=None,
                            full_name=None):
@@ -1839,6 +1900,11 @@ class MySQLCompiler(compiler.DefaultCompiler):
         sql_operators.concat_op: lambda x, y: "concat(%s, %s)" % (x, y),
         sql_operators.mod: '%%'
     })
+    functions = compiler.DefaultCompiler.functions.copy()
+    functions.update ({
+        sql_functions.random: 'rand%(expr)s'
+        })
+
 
     def visit_typeclause(self, typeclause):
         type_ = typeclause.type.dialect_impl(self.dialect)
@@ -1956,7 +2022,8 @@ class MySQLSchemaGenerator(compiler.SchemaGenerator):
         """Builds column DDL."""
 
         colspec = [self.preparer.format_column(column),
-                   column.type.dialect_impl(self.dialect, _for_ddl=True).get_col_spec()]
+                   column.type.dialect_impl(self.dialect,
+                                            _for_ddl=column).get_col_spec()]
 
         default = self.get_column_default_string(column)
         if default is not None:
@@ -2067,9 +2134,7 @@ class MySQLSchemaReflector(object):
             else:
                 type_, spec = self.parse_constraints(line)
                 if type_ is None:
-                    warnings.warn(
-                        RuntimeWarning("Unknown schema content: %s" %
-                                       repr(line)))
+                    util.warn("Unknown schema content: %r" % line)
                 elif type_ == 'key':
                     keys.append(spec)
                 elif type_ == 'constraint':
@@ -2097,12 +2162,10 @@ class MySQLSchemaReflector(object):
     def _add_column(self, table, line, charset, only=None):
         spec = self.parse_column(line)
         if not spec:
-            warnings.warn(RuntimeWarning(
-                "Unknown column definition %s" % line))
+            util.warn("Unknown column definition %r" % line)
             return
         if not spec['full']:
-            warnings.warn(RuntimeWarning(
-                "Incomplete reflection of column definition %s" % line))
+            util.warn("Incomplete reflection of column definition %r" % line)
 
         name, type_, args, notnull = \
               spec['name'], spec['coltype'], spec['arg'], spec['notnull']
@@ -2120,9 +2183,8 @@ class MySQLSchemaReflector(object):
         try:
             col_type = ischema_names[type_]
         except KeyError:
-            warnings.warn(RuntimeWarning(
-                "Did not recognize type '%s' of column '%s'" %
-                (type_, name)))
+            util.warn("Did not recognize type '%s' of column '%s'" %
+                      (type_, name))
             col_type = sqltypes.NullType
 
         # Column type positional arguments eg. varchar(32)
@@ -2160,7 +2222,7 @@ class MySQLSchemaReflector(object):
         default = spec.get('default', None)
         if default is not None and default != 'NULL':
             # Defaults should be in the native charset for the moment
-            default = default.decode(charset)
+            default = default.encode(charset)
             if type_ == 'timestamp':
                 # can't be NULL for TIMESTAMPs
                 if (default[0], default[-1]) != ("'", "'"):
