@@ -12,9 +12,9 @@ higher-level statement-construction, connection-management, execution
 and result contexts.
 """
 
-import StringIO, sys
+import inspect, StringIO, sys
 from sqlalchemy import exceptions, schema, util, types, logging
-from sqlalchemy.sql import expression, visitors
+from sqlalchemy.sql import expression
 
 
 class Dialect(object):
@@ -96,8 +96,8 @@ class Dialect(object):
       This is used to apply types to result sets based on the DB-API
       types present in cursor.description; it only takes effect for
       result sets against textual statements where no explicit
-      typemap was present.  
-      
+      typemap was present.
+
     """
 
     def create_connect_args(self, url):
@@ -317,14 +317,14 @@ class ExecutionContext(object):
 
     should_autocommit
       True if the statement is a "committable" statement
-      
+
     returns_rows
       True if the statement should return result rows
 
     postfetch_cols
      a list of Column objects for which a server-side default
      or inline SQL expression value was fired off.  applies to inserts and updates.
-      
+
     The Dialect should provide an ExecutionContext via the
     create_execution_context() method.  The `pre_exec` and `post_exec`
     methods will be called for compiled statements.
@@ -375,9 +375,9 @@ class ExecutionContext(object):
 
     def should_autocommit_compiled(self, compiled):
         """return True if the given Compiled object refers to a "committable" statement."""
-        
+
         raise NotImplementedError()
-        
+
     def should_autocommit_text(self, statement):
         """Parse the given textual statement and return True if it refers to a "committable" statement"""
 
@@ -411,7 +411,7 @@ class ExecutionContext(object):
         raise NotImplementedError()
 
     def lastrow_has_defaults(self):
-        """Return True if the last INSERT or UPDATE row contained 
+        """Return True if the last INSERT or UPDATE row contained
         inlined or database-side defaults.
         """
 
@@ -436,7 +436,7 @@ class Compiled(object):
 
         dialect
           ``Dialect`` to compile against.
-          
+
         statement
           ``ClauseElement`` to be compiled.
 
@@ -452,28 +452,26 @@ class Compiled(object):
         self.column_keys = column_keys
         self.bind = bind
         self.can_execute = statement.supports_execution()
-    
+
     def compile(self):
         """Produce the internal string representation of this element."""
-        
+
         raise NotImplementedError()
-        
+
     def __str__(self):
         """Return the string text of the generated SQL statement."""
 
         raise NotImplementedError()
 
     def get_params(self, **params):
-        """Use construct_params().  (supports unicode names)
-        """
-
+        """Use construct_params().  (supports unicode names)"""
         return self.construct_params(params)
-    get_params = util.deprecated(get_params)
+    get_params = util.deprecated()(get_params)
 
     def construct_params(self, params):
         """Return the bind params for this compiled object.
 
-        `params` is a dict of string/object pairs whos 
+        `params` is a dict of string/object pairs whos
         values will override bind values compiled in
         to the statement.
         """
@@ -484,7 +482,7 @@ class Compiled(object):
 
         e = self.bind
         if e is None:
-            raise exceptions.InvalidRequestError("This Compiled object is not bound to any Engine or Connection.")
+            raise exceptions.UnboundExecutionError("This Compiled object is not bound to any Engine or Connection.")
         return e._execute_compiled(self, multiparams, params)
 
     def scalar(self, *multiparams, **params):
@@ -545,7 +543,7 @@ class Connection(Connectable):
         self.__invalid = False
 
     def _branch(self):
-        """Return a new Connection which references this Connection's 
+        """Return a new Connection which references this Connection's
         engine and connection; but does not have close_with_result enabled,
         and also whose close() method does nothing.
 
@@ -556,22 +554,22 @@ class Connection(Connectable):
 
     def dialect(self):
         "Dialect used by this Connection."
-        
+
         return self.engine.dialect
     dialect = property(dialect)
-    
+
     def closed(self):
         """return True if this connection is closed."""
-        
+
         return not self.__invalid and '_Connection__connection' not in self.__dict__
     closed = property(closed)
-    
+
     def invalidated(self):
         """return True if this connection was invalidated."""
-        
+
         return self.__invalid
     invalidated = property(invalidated)
-    
+
     def connection(self):
         "The underlying DB-API connection managed by this Connection."
 
@@ -586,7 +584,7 @@ class Connection(Connectable):
                 return self.__connection
             raise exceptions.InvalidRequestError("This Connection is closed")
     connection = property(connection)
-    
+
     def should_close_with_result(self):
         """Indicates if this Connection should be closed when a corresponding
         ResultProxy is closed; this is essentially an auto-release mode.
@@ -627,17 +625,17 @@ class Connection(Connectable):
 
         The underlying DB-API connection is literally closed (if
         possible), and is discarded.  Its source connection pool will
-        typically lazilly create a new connection to replace it.
-        
+        typically lazily create a new connection to replace it.
+
         Upon the next usage, this Connection will attempt to reconnect
         to the pool with a new connection.
 
         Transactions in progress remain in an "opened" state (even though
-        the actual transaction is gone); these must be explicitly 
+        the actual transaction is gone); these must be explicitly
         rolled back before a reconnect on this Connection can proceed.  This
         is to prevent applications from accidentally continuing their transactional
         operations in a non-transactional state.
-        
+
         """
 
         if self.__connection.is_valid:
@@ -757,7 +755,7 @@ class Connection(Connectable):
         except Exception, e:
             self._handle_dbapi_exception(e, None, None, None)
             raise
-        
+
     def _savepoint_impl(self, name=None):
         if name is None:
             self.__savepoint_seq += 1
@@ -819,17 +817,19 @@ class Connection(Connectable):
         """Close this Connection."""
 
         try:
-            c = self.__connection
+            conn = self.__connection
         except AttributeError:
             return
         if not self.__branch:
-            self.__connection.close()
-        self.__connection = None
+            conn.close()
         self.__invalid = False
         del self.__connection
 
     def scalar(self, object, *multiparams, **params):
-        """Executes and returns the first column of the first row."""
+        """Executes and returns the first column of the first row.
+
+        The underlying result/cursor is closed after execution.
+        """
 
         return self.execute(object, *multiparams, **params).scalar()
 
@@ -857,11 +857,11 @@ class Connection(Connectable):
 
     def __distill_params(self, multiparams, params):
         """given arguments from the calling form *multiparams, **params, return a list
-        of bind parameter structures, usually a list of dictionaries.  
-        
-        in the case of 'raw' execution which accepts positional parameters, 
+        of bind parameter structures, usually a list of dictionaries.
+
+        in the case of 'raw' execution which accepts positional parameters,
         it may be a list of tuples or lists."""
-        
+
         if multiparams is None or len(multiparams) == 0:
             if params:
                 return [params]
@@ -915,6 +915,13 @@ class Connection(Connectable):
         else:
             self._cursor_execute(context.cursor, context.statement, context.parameters[0], context=context)
 
+    def _execute_ddl(self, ddl, params, multiparams):
+        if params:
+            schema_item, params = params[0], params[1:]
+        else:
+            schema_item = None
+        return ddl(None, schema_item, self, *params, **multiparams)
+
     def _handle_dbapi_exception(self, e, statement, parameters, cursor):
         if getattr(self, '_reentrant_error', False):
             raise exceptions.DBAPIError.instance(None, None, e)
@@ -935,7 +942,7 @@ class Connection(Connectable):
             raise exceptions.DBAPIError.instance(statement, parameters, e, connection_invalidated=is_disconnect)
         finally:
             del self._reentrant_error
-        
+
     def __create_execution_context(self, **kwargs):
         try:
             return self.engine.dialect.create_execution_context(connection=self, **kwargs)
@@ -965,11 +972,12 @@ class Connection(Connectable):
 
     # poor man's multimethod/generic function thingy
     executors = {
-        expression._Function : _execute_function,
-        expression.ClauseElement : execute_clauseelement,
-        Compiled : _execute_compiled,
-        schema.SchemaItem:_execute_default,
-        str.__mro__[-2] : _execute_text
+        expression._Function: _execute_function,
+        expression.ClauseElement: execute_clauseelement,
+        Compiled: _execute_compiled,
+        schema.SchemaItem: _execute_default,
+        schema.DDL: _execute_ddl,
+        basestring: _execute_text
     }
 
     def create(self, entity, **kwargs):
@@ -1003,7 +1011,7 @@ class Transaction(object):
         self._connection = connection
         self._parent = parent or self
         self._is_active = True
-    
+
     def connection(self):
         "The Connection object referenced by this Transaction"
         return self._connection
@@ -1059,7 +1067,7 @@ class RootTransaction(Transaction):
     def __init__(self, connection):
         super(RootTransaction, self).__init__(connection, None)
         self._connection._begin_impl()
-    
+
     def _do_rollback(self):
         self._connection._rollback_impl()
 
@@ -1070,7 +1078,7 @@ class NestedTransaction(Transaction):
     def __init__(self, connection, parent):
         super(NestedTransaction, self).__init__(connection, parent)
         self._savepoint = self._connection._savepoint_impl()
-    
+
     def _do_rollback(self):
         self._connection._rollback_to_savepoint_impl(self._savepoint, self._parent)
 
@@ -1083,16 +1091,16 @@ class TwoPhaseTransaction(Transaction):
         self._is_prepared = False
         self.xid = xid
         self._connection._begin_twophase_impl(self.xid)
-    
+
     def prepare(self):
         if not self._parent._is_active:
             raise exceptions.InvalidRequestError("This transaction is inactive")
         self._connection._prepare_twophase_impl(self.xid)
         self._is_prepared = True
-    
+
     def _do_rollback(self):
         self._connection._rollback_twophase_impl(self.xid, self._is_prepared)
-    
+
     def commit(self):
         self._connection._commit_twophase_impl(self.xid, self._is_prepared)
 
@@ -1112,15 +1120,15 @@ class Engine(Connectable):
 
     def name(self):
         "String name of the [sqlalchemy.engine#Dialect] in use by this ``Engine``."
-        
+
         return sys.modules[self.dialect.__module__].descriptor()['name']
     name = property(name)
-    
+
     echo = logging.echo_property()
-    
+
     def __repr__(self):
         return 'Engine(%s)' % str(self.url)
-    
+
     def dispose(self):
         self.pool.dispose()
         self.pool = self.pool.recreate()
@@ -1229,7 +1237,7 @@ class Engine(Connectable):
         """
 
         return Connection(self, self.pool.connect(), close_with_result=close_with_result, **kwargs)
-    
+
     def table_names(self, schema=None, connection=None):
         """Return a list of all table names available in the database.
 
@@ -1306,7 +1314,7 @@ class RowProxy(object):
 
     def __len__(self):
         return len(self.__row)
-        
+
     def __iter__(self):
         for i in xrange(len(self.__row)):
             yield self.__parent._get_col(self.__row, i)
@@ -1316,6 +1324,9 @@ class RowProxy(object):
                 (other == tuple([self.__parent._get_col(self.__row, key)
                                  for key in xrange(len(self.__row))])))
 
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
     def __repr__(self):
         return repr(tuple(self))
 
@@ -1400,11 +1411,11 @@ class ResultProxy(object):
         else:
             return self.context.get_rowcount()
     rowcount = property(rowcount)
-    
+
     def lastrowid(self):
         return self.cursor.lastrowid
     lastrowid = property(lastrowid)
-    
+
     def out_parameters(self):
         return self.context.out_parameters
     out_parameters = property(out_parameters)
@@ -1420,14 +1431,14 @@ class ResultProxy(object):
 
             for i, item in enumerate(metadata):
                 colname = item[0].decode(self.dialect.encoding)
-                
+
                 if '.' in colname:
                     # sqlite will in some circumstances prepend table name to colnames, so strip
                     origname = colname
                     colname = colname.split('.')[-1]
                 else:
                     origname = None
-                    
+
                 if self.context.result_map:
                     try:
                         (name, obj, type_) = self.context.result_map[colname.lower()]
@@ -1440,12 +1451,12 @@ class ResultProxy(object):
 
                 if self.__props.setdefault(name.lower(), rec) is not rec:
                     self.__props[name.lower()] = (type_, self.__ambiguous_processor(name), 0)
-                
+
                 # store the "origname" if we truncated (sqlite only)
                 if origname:
                     if self.__props.setdefault(origname.lower(), rec) is not rec:
                         self.__props[origname.lower()] = (type_, self.__ambiguous_processor(origname), 0)
-                    
+
                 self.__keys.append(colname)
                 self.__props[i] = rec
                 if obj:
@@ -1463,18 +1474,19 @@ class ResultProxy(object):
             """Given a key, which could be a ColumnElement, string, etc.,
             matches it to the appropriate key we got from the result set's
             metadata; then cache it locally for quick re-access."""
-            
+
             if isinstance(key, basestring):
                 key = key.lower()
-            
             try:
                 rec = props[key]
             except KeyError:
                 # fallback for targeting a ColumnElement to a textual expression
+                # this is a rare use case which only occurs when matching text()
+                # constructs to ColumnElements
                 if isinstance(key, expression.ColumnElement):
-                    if key._label.lower() in props:
+                    if key._label and key._label.lower() in props:
                         return props[key._label.lower()]
-                    elif key.name.lower() in props:
+                    elif hasattr(key, 'name') and key.name.lower() in props:
                         return props[key.name.lower()]
                 raise exceptions.NoSuchColumnError("Could not locate column in row for column '%s'" % (str(key)))
 
@@ -1554,13 +1566,16 @@ class ResultProxy(object):
         """
 
         return self.context.lastrow_has_defaults()
-    
+
     def postfetch_cols(self):
         """Return ``postfetch_cols()`` from the underlying ExecutionContext.
 
         See ExecutionContext for details.
         """
         return self.context.postfetch_cols
+    
+    def prefetch_cols(self):
+        return self.context.prefetch_cols
         
     def supports_sane_rowcount(self):
         """Return ``supports_sane_rowcount`` from the dialect.
@@ -1585,7 +1600,7 @@ class ResultProxy(object):
                 return tuple([self._get_col(row, i) for i in xrange(*indices)])
             else:
                 raise
-                
+
         if processor:
             return processor(row[index])
         else:
@@ -1593,40 +1608,59 @@ class ResultProxy(object):
 
     def _fetchone_impl(self):
         return self.cursor.fetchone()
-        
+
     def _fetchmany_impl(self, size=None):
         return self.cursor.fetchmany(size)
-        
+
     def _fetchall_impl(self):
         return self.cursor.fetchall()
 
     def fetchall(self):
         """Fetch all rows, just like DB-API ``cursor.fetchall()``."""
 
-        l = [self._process_row(self, row) for row in self._fetchall_impl()]
-        self.close()
-        return l
+        try:
+            process_row = self._process_row
+            l = [process_row(self, row) for row in self._fetchall_impl()]
+            self.close()
+            return l
+        except Exception, e:
+            self.connection._handle_dbapi_exception(e, None, None, self.cursor)
+            raise
 
     def fetchmany(self, size=None):
         """Fetch many rows, just like DB-API ``cursor.fetchmany(size=cursor.arraysize)``."""
 
-        l = [self._process_row(self, row) for row in self._fetchmany_impl(size)]
-        if len(l) == 0:
-            self.close()
-        return l
+        try:
+            process_row = self._process_row
+            l = [process_row(self, row) for row in self._fetchmany_impl(size)]
+            if len(l) == 0:
+                self.close()
+            return l
+        except Exception, e:
+            self.connection._handle_dbapi_exception(e, None, None, self.cursor)
+            raise
 
     def fetchone(self):
         """Fetch one row, just like DB-API ``cursor.fetchone()``."""
-        row = self._fetchone_impl()
-        if row is not None:
-            return self._process_row(self, row)
-        else:
-            self.close()
-            return None
+        try:
+            row = self._fetchone_impl()
+            if row is not None:
+                return self._process_row(self, row)
+            else:
+                self.close()
+                return None
+        except Exception, e:
+            self.connection._handle_dbapi_exception(e, None, None, self.cursor)
+            raise
 
     def scalar(self):
         """Fetch the first column of the first row, and close the result set."""
-        row = self._fetchone_impl()
+        try:
+            row = self._fetchone_impl()
+        except Exception, e:
+            self.connection._handle_dbapi_exception(e, None, None, self.cursor)
+            raise
+            
         try:
             if row is not None:
                 return self._process_row(self, row)[0]
@@ -1774,6 +1808,7 @@ class DefaultRunner(schema.SchemaVisitor):
     def __init__(self, context):
         self.context = context
         self.dialect = context.dialect
+        self.cursor = context.cursor
 
     def get_column_default(self, column):
         if column.default is not None:
@@ -1807,14 +1842,16 @@ class DefaultRunner(schema.SchemaVisitor):
         conn = self.context.connection
         c = expression.select([default.arg]).compile(bind=conn)
         return conn._execute_compiled(c).scalar()
-    
+
     def execute_string(self, stmt, params=None):
         """execute a string statement, using the raw cursor,
         and return a scalar result."""
         conn = self.context._connection
-        conn._cursor_execute(self.context.cursor, stmt, params)
-        return self.context.cursor.fetchone()[0]
-        
+        if isinstance(stmt, unicode) and not self.dialect.supports_unicode_statements:
+            stmt = stmt.encode(self.dialect.encoding)
+        conn._cursor_execute(self.cursor, stmt, params)
+        return self.cursor.fetchone()[0]
+
     def visit_column_onupdate(self, onupdate):
         if isinstance(onupdate.arg, expression.ClauseElement):
             return self.exec_default_sql(onupdate)
@@ -1830,3 +1867,27 @@ class DefaultRunner(schema.SchemaVisitor):
             return default.arg(self.context)
         else:
             return default.arg
+
+
+def connection_memoize(key):
+    """Decorator, memoize a function in a connection.info stash.
+
+    Only applicable to functions which take no arguments other than a
+    connection.  The memo will be stored in ``connection.info[key]``.
+
+    """
+    def decorate(fn):
+        spec = inspect.getargspec(fn)
+        assert len(spec[0]) == 2
+        assert spec[0][1] == 'connection'
+        assert spec[1:3] == (None, None)
+
+        def decorated(self, connection):
+            try:
+                return connection.info[key]
+            except KeyError:
+                connection.info[key] = val = fn(self, connection)
+                return val
+
+        return util.function_named(decorated, fn.__name__)
+    return decorate
