@@ -127,6 +127,13 @@ class SessionExtension(object):
         state.  An actual commit() may or may not have occured, depending on whether or not
         the flush started its own transaction or participated in a larger transaction.
         """
+    
+    def after_begin(self, session, transaction, connection):
+        """Execute after a transaction is begun on a connection
+        
+        `transaction` is the SessionTransaction. This method is called after an
+        engine level transaction is begun on a connection.
+        """
 
 class SessionTransaction(object):
     """Represents a Session-level Transaction.
@@ -144,110 +151,163 @@ class SessionTransaction(object):
 
     def __init__(self, session, parent=None, autoflush=True, nested=False):
         self.session = session
-        self.__connections = {}
-        self.__parent = parent
+        self._connections = {}
+        self._parent = parent
         self.autoflush = autoflush
         self.nested = nested
+        self._active = True
+        self._prepared = False
+
+    is_active = property(lambda s: s.session is not None and s._active)
+    
+    def _assert_is_active(self):
+        self._assert_is_open()
+        if not self._active:
+            raise exceptions.InvalidRequestError("The transaction is inactive due to a rollback in a subtransaction and should be closed")
+
+    def _assert_is_open(self):
+        if self.session is None:
+            raise exceptions.InvalidRequestError("The transaction is closed")
 
     def connection(self, bindkey, **kwargs):
+        self._assert_is_active()
         engine = self.session.get_bind(bindkey, **kwargs)
         return self.get_or_add(engine)
 
     def _begin(self, **kwargs):
+        self._assert_is_active()
         return SessionTransaction(self.session, self, **kwargs)
 
-    def add(self, bind):
-        if self.__parent is not None:
-            return self.__parent.add(bind)
+    def _iterate_parents(self, upto=None):
+        if self._parent is upto:
+            return (self,)
+        else:
+            if self._parent is None:
+                raise exceptions.InvalidRequestError("Transaction %s is not on the active transaction list" % upto)
+            return (self,) + self._parent._iterate_parents(upto)
 
-        if bind.engine in self.__connections:
+    def add(self, bind):
+        self._assert_is_active()
+        if self._parent is not None and not self.nested:
+            return self._parent.add(bind)
+
+        if bind.engine in self._connections:
             raise exceptions.InvalidRequestError("Session already has a Connection associated for the given %sEngine" % (isinstance(bind, engine.Connection) and "Connection's " or ""))
         return self.get_or_add(bind)
 
-    def _connection_dict(self):
-        if self.__parent is not None and not self.nested:
-            return self.__parent._connection_dict()
-        else:
-            return self.__connections
-
     def get_or_add(self, bind):
-        if self.__parent is not None:
+        self._assert_is_active()
+        
+        if bind in self._connections:
+            return self._connections[bind][0]
+        
+        if self._parent is not None:
+            conn = self._parent.get_or_add(bind)
             if not self.nested:
-                return self.__parent.get_or_add(bind)
-
-            if bind in self.__connections:
-                return self.__connections[bind][0]
-
-            conn_dict = self.__parent._connection_dict()
-            if bind in conn_dict:
-                (conn, trans, autoclose) = conn_dict[bind]
-                self.__connections[conn] = self.__connections[bind.engine] = (conn, conn.begin_nested(), autoclose)
                 return conn
-        elif bind in self.__connections:
-            return self.__connections[bind][0]
-
-        if not isinstance(bind, engine.Connection):
-            e = bind
-            c = bind.contextual_connect()
         else:
-            e = bind.engine
-            c = bind
-            if e in self.__connections:
-                raise exceptions.InvalidRequestError("Session already has a Connection associated for the given Connection's Engine")
-        if self.nested:
-            trans = c.begin_nested()
-        elif self.session.twophase:
-            trans = c.begin_twophase()
+            if isinstance(bind, engine.Connection):
+                conn = bind
+                if conn.engine in self._connections:
+                    raise exceptions.InvalidRequestError("Session already has a Connection associated for the given Connection's Engine")
+            else:
+                conn = bind.contextual_connect()
+
+        if self.session.twophase and self._parent is None:
+            transaction = conn.begin_twophase()
+        elif self.nested:
+            transaction = conn.begin_nested()
         else:
-            trans = c.begin()
-        self.__connections[c] = self.__connections[e] = (c, trans, c is not bind)
-        return self.__connections[c][0]
-
-    def commit(self):
-        if self.__parent is not None and not self.nested:
-            return self.__parent
-
+            transaction = conn.begin()
+        
+        self._connections[conn] = self._connections[conn.engine] = (conn, transaction, conn is not bind)
         if self.session.extension is not None:
-            self.session.extension.before_commit(self.session)
+            self.session.extension.after_begin(self.session, self, conn)
+        return conn
 
+    def prepare(self):
+        if self._parent is not None or not self.session.twophase:
+            raise exceptions.InvalidRequestError("Only root two phase transactions of can be prepared")
+        self._prepare_impl()
+        
+    def _prepare_impl(self):
+        self._assert_is_active()
+        if self.session.extension is not None and (self._parent is None or self.nested):
+            self.session.extension.before_commit(self.session)
+        
+        if self.session.transaction is not self:
+            for subtransaction in self.session.transaction._iterate_parents(upto=self):
+                subtransaction.commit()
+            
         if self.autoflush:
             self.session.flush()
+        
+        if self._parent is None and self.session.twophase:
+            try:
+                for t in util.Set(self._connections.values()):
+                    t[1].prepare()
+            except:
+                self.rollback()
+                raise
+        
+        self._deactivate()
+        self._prepared = True
+    
+    def commit(self):
+        self._assert_is_open()
+        if not self._prepared:
+            self._prepare_impl()
+        
+        if self._parent is None or self.nested:
+            for t in util.Set(self._connections.values()):
+                t[1].commit()
 
-        if self.session.twophase:
-            for t in util.Set(self.__connections.values()):
-                t[1].prepare()
-
-        for t in util.Set(self.__connections.values()):
-            t[1].commit()
-
-        if self.session.extension is not None:
-            self.session.extension.after_commit(self.session)
+            if self.session.extension is not None:
+                self.session.extension.after_commit(self.session)
 
         self.close()
-        return self.__parent
+        return self._parent
 
     def rollback(self):
-        if self.__parent is not None and not self.nested:
-            return self.__parent.rollback()
+        self._assert_is_open()
+        
+        if self.session.transaction is not self:
+            for subtransaction in self.session.transaction._iterate_parents(upto=self):
+                subtransaction.close()
+        
+        if self.is_active or self._prepared:
+            for transaction in self._iterate_parents():
+                if transaction._parent is None or transaction.nested:
+                    transaction._rollback_impl()
+                    transaction._deactivate()
+                    break
+                else:
+                    transaction._deactivate()
 
-        for t in util.Set(self.__connections.values()):
+        self.close()
+        return self._parent
+    
+    def _rollback_impl(self):
+        for t in util.Set(self._connections.values()):
             t[1].rollback()
 
         if self.session.extension is not None:
             self.session.extension.after_rollback(self.session)
 
-        self.close()
-        return self.__parent
+    def _deactivate(self):
+        self._active = False
 
     def close(self):
-        if self.__parent is not None:
-            return
-        for t in util.Set(self.__connections.values()):
-            if t[2]:
-                t[0].close()
-            else:
-                t[1].close()
-        self.session.transaction = None
+        self.session.transaction = self._parent
+        if self._parent is None:
+            for connection, transaction, autoclose in util.Set(self._connections.values()):
+                if autoclose:
+                    connection.close()
+                else:
+                    transaction.close()
+        self._deactivate()
+        self.session = None
+        self._connections = None
 
     def __enter__(self):
         return self
@@ -256,7 +316,11 @@ class SessionTransaction(object):
         if self.session.transaction is None:
             return
         if type is None:
-            self.commit()
+            try:
+                self.commit()
+            except:
+                self.rollback()
+                raise
         else:
             self.rollback()
 
@@ -458,7 +522,7 @@ class Session(object):
         if self.transaction is None:
             pass
         else:
-            self.transaction = self.transaction.rollback()
+            self.transaction.rollback()
         # TODO: we can rollback attribute values.  however
         # we would want to expand attributes.py to be able to save *two* rollback points, one to the
         # last flush() and the other to when the object first entered the transaction.
@@ -484,15 +548,31 @@ class Session(object):
         if self.transaction is None:
             if self.transactional:
                 self.begin()
-                self.transaction = self.transaction.commit()
             else:
                 raise exceptions.InvalidRequestError("No transaction is begun.")
-        else:
-            self.transaction = self.transaction.commit()
+
+        self.transaction.commit()
         if self.transaction is None and self.transactional:
             self.begin()
+    
+    def prepare(self):
+        """Prepare the current transaction in progress for two phase commit.
 
-    def connection(self, mapper=None, **kwargs):
+        If no transaction is in progress, this method raises
+        an InvalidRequestError.
+
+        Only root transactions of two phase sessions can be prepared. If the current transaction is
+        not such, an InvalidRequestError is raised.
+        """
+        if self.transaction is None:
+            if self.transactional:
+                self.begin()
+            else:
+                raise exceptions.InvalidRequestError("No transaction is begun.")
+
+        self.transaction.prepare()
+
+    def connection(self, mapper=None, clause=None, instance=None):
         """Return a ``Connection`` corresponding to this session's
         transactional context, if any.
 
@@ -510,7 +590,7 @@ class Session(object):
         subclass takes a different get_bind() argument signature.
         """
 
-        return self.__connection(self.get_bind(mapper, **kwargs))
+        return self.__connection(self.get_bind(mapper, clause, instance))
 
     def __connection(self, engine, **kwargs):
         if self.transaction is not None:
@@ -518,28 +598,38 @@ class Session(object):
         else:
             return engine.contextual_connect(**kwargs)
 
-    def execute(self, clause, params=None, mapper=None, **kwargs):
-        """Using the given mapper to identify the appropriate ``Engine``
-        or ``Connection`` to be used for statement execution, execute the
-        given ``ClauseElement`` using the provided parameter dictionary.
+    def execute(self, clause, params=None, mapper=None, instance=None):
+        """Execute the given clause, using the current transaction (if any).
 
-        Return a ``ResultProxy`` corresponding to the execution's results.
-
-        If this method allocates a new ``Connection`` for the operation,
-        then the ``ResultProxy`` 's ``close()`` method will release the
-        resources of the underlying ``Connection``.
+        Returns a ``ResultProxy`` corresponding to the execution's results.
+        
+        clause
+            a ClauseElement (i.e. select(), text(), etc.) or 
+            string SQL statement to be executed
+            
+        params 
+            a dictionary of bind parameters.
+        
+        mapper
+            a mapped class or Mapper instance which may be needed
+            in order to locate the proper bind.  This is typically
+            if the Session is not directly bound to a single engine.
+            
+        instance
+            used by some Query operations to further identify
+            the proper bind, in the case of ShardedSession.
+            
         """
-
-        engine = self.get_bind(mapper, clause=clause, **kwargs)
+        engine = self.get_bind(mapper, clause=clause, instance=instance)
 
         return self.__connection(engine, close_with_result=True).execute(clause, params or {})
 
-    def scalar(self, clause, params=None, mapper=None, **kwargs):
+    def scalar(self, clause, params=None, mapper=None, instance=None):
         """Like execute() but return a scalar result."""
 
-        engine = self.get_bind(mapper, clause=clause)
+        engine = self.get_bind(mapper, clause=clause, instance=instance)
 
-        return self.__connection(engine, close_with_result=True).scalar(clause, params or {}, **kwargs)
+        return self.__connection(engine, close_with_result=True).scalar(clause, params or {})
 
     def close(self):
         """Close this Session.
@@ -554,7 +644,8 @@ class Session(object):
 
         self.clear()
         if self.transaction is not None:
-            self.transaction.close()
+            for transaction in self.transaction._iterate_parents():
+                transaction.close()
         if self.transactional:
             # note this doesnt use any connection resources
             self.begin()
@@ -572,7 +663,7 @@ class Session(object):
         This is equivalent to calling ``expunge()`` for all objects in
         this ``Session``.
         """
-
+        
         for instance in self:
             self._unattach(instance)
         self.uow = unitofwork.UnitOfWork(self)
@@ -603,28 +694,29 @@ class Session(object):
 
         self.__binds[table] = bind
 
-    def get_bind(self, mapper, clause=None, **kwargs):
+    def get_bind(self, mapper, clause=None, instance=None):
         """Return an engine corresponding to the given arguments.
 
         mapper
-            mapper relative to the desired operation
+            mapper relative to the desired operation.
 
         clause
             a ClauseElement which is to be executed.  if
             mapper is not present, this may be used to locate
             Table objects, which are then associated with mappers
             which have associated binds.
-
-        \**kwargs
-            Subclasses (i.e. ShardedSession) may add additional arguments
-            to get_bind() which are passed through here.
+        
+        instance
+            an ORM mapped instance which may be used to further
+            locate the correct bind.  This is currently used by 
+            the ShardedSession subclass.
+            
         """
-
         if mapper is None and clause is None:
             if self.bind is not None:
                 return self.bind
             else:
-                raise exceptions.InvalidRequestError("This session is unbound to any Engine or Connection; specify a mapper to get_bind()")
+                raise exceptions.UnboundExecutionError("This session is unbound to any Engine or Connection; specify a mapper to get_bind()")
 
         elif len(self.__binds):
             if mapper is not None:
@@ -641,8 +733,10 @@ class Session(object):
 
         if self.bind is not None:
             return self.bind
+        elif isinstance(clause, sql.expression.ClauseElement) and clause.bind is not None:
+            return clause.bind
         elif mapper is None:
-            raise exceptions.InvalidRequestError("Could not locate any mapper associated with SQL expression")
+            raise exceptions.UnboundExecutionError("Could not locate any mapper associated with SQL expression")
         else:
             if isinstance(mapper, type):
                 mapper = _class_mapper(mapper)
@@ -650,14 +744,14 @@ class Session(object):
                 mapper = mapper.compile()
             e = mapper.mapped_table.bind
             if e is None:
-                raise exceptions.InvalidRequestError("Could not locate any Engine or Connection bound to mapper '%s'" % str(mapper))
+                raise exceptions.UnboundExecutionError("Could not locate any Engine or Connection bound to mapper '%s'" % str(mapper))
             return e
 
     def query(self, mapper_or_class, *addtl_entities, **kwargs):
         """Return a new ``Query`` object corresponding to this ``Session`` and
         the mapper, or the classes' primary mapper.
-        """
 
+        """
         entity_name = kwargs.pop('entity_name', None)
 
         if isinstance(mapper_or_class, type):
@@ -669,15 +763,6 @@ class Session(object):
             q = q.add_entity(ent)
         return q
 
-    def sql(self):
-        class SQLProxy(object):
-            def __getattr__(self, key):
-                def call(*args, **kwargs):
-                    kwargs[engine] = self.engine
-                    return getattr(sql, key)(*args, **kwargs)
-
-    sql = property(sql)
-
     def _autoflush(self):
         if self.autoflush and (self.transaction is None or self.transaction.autoflush):
             self.flush()
@@ -686,10 +771,18 @@ class Session(object):
         """Flush all the object modifications present in this session
         to the database.
 
-        `objects` is a list or tuple of objects specifically to be
+        `objects` is a collection or iterator of objects specifically to be
         flushed; if ``None``, all new and modified objects are flushed.
-        """
 
+        """
+        if objects is not None:
+            try:
+                if not len(objects):
+                    return
+            except TypeError:
+                objects = list(objects)
+                if not objects:
+                    return
         self.uow.flush(self, objects)
 
     def get(self, class_, ident, **kwargs):
@@ -749,7 +842,14 @@ class Session(object):
 
         if self.query(_object_mapper(instance))._get(instance._instance_key, refresh_instance=instance._state, only_load_props=attribute_names) is None:
             raise exceptions.InvalidRequestError("Could not refresh instance '%s'" % mapperutil.instance_str(instance))
-
+    
+    def expire_all(self):
+        """Expires all persistent instances within this Session.  
+        
+        """
+        for state in self.identity_map.all_states():
+            _expire_state(state, None)
+        
     def expire(self, instance, attribute_names=None):
         """Expire the attributes on the given instance.
 
@@ -758,13 +858,6 @@ class Session(object):
         to the database which will refresh all attributes with their
         current value.
 
-        Lazy-loaded relational attributes will remain lazily loaded, so that
-        triggering one will incur the instance-wide refresh operation, followed
-        immediately by the lazy load of that attribute.
-
-        Eagerly-loaded relational attributes will eagerly load within the
-        single refresh operation.
-
         The ``attribute_names`` argument is an iterable collection
         of attribute names indicating a subset of attributes to be
         expired.
@@ -878,7 +971,8 @@ class Session(object):
         """
 
         if _recursive is None:
-            _recursive = {}  #TODO: this should be an IdentityDict
+            _recursive = {}  # TODO: this should be an IdentityDict for instances, but will need a separate
+                             # dict for PropertyLoader tuples
         if entity_name is not None:
             mapper = _class_mapper(instance.__class__, entity_name=entity_name)
         else:
@@ -888,8 +982,12 @@ class Session(object):
 
         key = getattr(instance, '_instance_key', None)
         if key is None:
-            merged = attributes.new_instance(mapper.class_)
-        else:
+            if dont_load:
+                raise exceptions.InvalidRequestError("merge() with dont_load=True option does not support objects transient (i.e. unpersisted) objects.  flush() all changes on mapped instances before merging with dont_load=True.")
+            key = mapper.identity_key_from_instance(instance)
+
+        merged = None
+        if key:
             if key in self.identity_map:
                 merged = self.identity_map[key]
             elif dont_load:
@@ -902,15 +1000,19 @@ class Session(object):
                 self._update_impl(merged, entity_name=mapper.entity_name)
             else:
                 merged = self.get(mapper.class_, key[1])
-                if merged is None:
-                    raise exceptions.AssertionError("Instance %s has an instance key but is not persisted" % mapperutil.instance_str(instance))
+        
+        if merged is None:
+            merged = attributes.new_instance(mapper.class_)
+            self.save(merged, entity_name=mapper.entity_name)
+            
         _recursive[instance] = merged
+        
         for prop in mapper.iterate_properties:
             prop.merge(self, instance, merged, dont_load, _recursive)
-        if key is None:
-            self.save(merged, entity_name=mapper.entity_name)
-        elif dont_load:
-            merged._state.commit_all()
+            
+        if dont_load:
+            merged._state.commit_all()  # remove any history
+
         return merged
 
     def identity_key(cls, *args, **kwargs):
@@ -1044,7 +1146,8 @@ class Session(object):
         ``Session``.
         """
 
-        return instance in self
+        if instance not in self:
+            raise exceptions.InvalidRequestError("Instance '%s' is not persistent within this Session" % mapperutil.instance_str(instance))
 
     def __contains__(self, instance):
         """Return True if the given instance is associated with this session.
@@ -1141,7 +1244,7 @@ def object_session(instance):
     hashkey = getattr(instance, '_sa_session_id', None)
     if hashkey is not None:
         sess = _sessions.get(hashkey)
-        if instance in sess:
+        if sess is not None and instance in sess:
             return sess
     return None
 
