@@ -1,13 +1,15 @@
-import testbase
+import testenv; testenv.configure_for_tests()
 import datetime
 from sqlalchemy import *
+from sqlalchemy.orm import *
 from sqlalchemy import exceptions
 from sqlalchemy.databases import postgres
 from sqlalchemy.engine.strategies import MockEngineStrategy
 from testlib import *
 from sqlalchemy.sql import table, column
 
-class SequenceTest(SQLCompileTest):
+
+class SequenceTest(TestBase, AssertsCompiledSQL):
     def test_basic(self):
         seq = Sequence("my_seq_no_schema")
         dialect = postgres.PGDialect()
@@ -19,13 +21,13 @@ class SequenceTest(SQLCompileTest):
         seq = Sequence("My_Seq", schema="Some_Schema")
         assert dialect.identifier_preparer.format_sequence(seq) == '"Some_Schema"."My_Seq"'
 
-class CompileTest(SQLCompileTest):
+class CompileTest(TestBase, AssertsCompiledSQL):
     def test_update_returning(self):
         dialect = postgres.dialect()
         table1 = table('mytable',
             column('myid', Integer),
-            column('name', String),
-            column('description', String),
+            column('name', String(128)),
+            column('description', String(128)),
         )
 
         u = update(table1, values=dict(name='foo'), postgres_returning=[table1.c.myid, table1.c.name])
@@ -42,8 +44,8 @@ class CompileTest(SQLCompileTest):
         dialect = postgres.dialect()
         table1 = table('mytable',
             column('myid', Integer),
-            column('name', String),
-            column('description', String),
+            column('name', String(128)),
+            column('description', String(128)),
         )
 
         i = insert(table1, values=dict(name='foo'), postgres_returning=[table1.c.myid, table1.c.name])
@@ -56,12 +58,12 @@ class CompileTest(SQLCompileTest):
         i = insert(table1, values=dict(name='foo'), postgres_returning=[func.length(table1.c.name)])
         self.assert_compile(i, "INSERT INTO mytable (name) VALUES (%(name)s) RETURNING length(mytable.name)", dialect=dialect)
 
-class ReturningTest(AssertMixin):
+class ReturningTest(TestBase, AssertsExecutionResults):
     __only_on__ = 'postgres'
 
     @testing.exclude('postgres', '<', (8, 2))
     def test_update_returning(self):
-        meta = MetaData(testbase.db)
+        meta = MetaData(testing.db)
         table = Table('tables', meta,
             Column('id', Integer, primary_key=True),
             Column('persons', Integer),
@@ -81,7 +83,7 @@ class ReturningTest(AssertMixin):
 
     @testing.exclude('postgres', '<', (8, 2))
     def test_insert_returning(self):
-        meta = MetaData(testbase.db)
+        meta = MetaData(testing.db)
         table = Table('tables', meta,
             Column('id', Integer, primary_key=True),
             Column('persons', Integer),
@@ -102,18 +104,18 @@ class ReturningTest(AssertMixin):
             result3 = table.insert(postgres_returning=[(table.c.id*2).label('double_id')]).execute({'persons': 4, 'full': False})
             self.assertEqual([dict(row) for row in result3], [{'double_id':8}])
 
-            result4 = testbase.db.execute('insert into tables (id, persons, "full") values (5, 10, true) returning persons')
+            result4 = testing.db.execute('insert into tables (id, persons, "full") values (5, 10, true) returning persons')
             self.assertEqual([dict(row) for row in result4], [{'persons': 10}])
         finally:
             table.drop()
 
 
-class InsertTest(AssertMixin):
+class InsertTest(TestBase, AssertsExecutionResults):
     __only_on__ = 'postgres'
 
     def setUpAll(self):
         global metadata
-        metadata = MetaData(testbase.db)
+        metadata = MetaData(testing.db)
 
     def tearDown(self):
         metadata.drop_all()
@@ -183,7 +185,7 @@ class InsertTest(AssertMixin):
         # note that the test framework doesnt capture the "preexecute" of a seqeuence
         # or default.  we just see it in the bind params.
 
-        self.assert_sql(testbase.db, go, [], with_sequences=[
+        self.assert_sql(testing.db, go, [], with_sequences=[
             (
                 "INSERT INTO testtable (id, data) VALUES (:id, :data)",
                 {'id':30, 'data':'d1'}
@@ -224,7 +226,7 @@ class InsertTest(AssertMixin):
 
         # test the same series of events using a reflected
         # version of the table
-        m2 = MetaData(testbase.db)
+        m2 = MetaData(testing.db)
         table = Table(table.name, m2, autoload=True)
 
         def go():
@@ -236,7 +238,7 @@ class InsertTest(AssertMixin):
             table.insert(inline=True).execute({'id':33, 'data':'d7'})
             table.insert(inline=True).execute({'data':'d8'})
 
-        self.assert_sql(testbase.db, go, [], with_sequences=[
+        self.assert_sql(testing.db, go, [], with_sequences=[
             (
                 "INSERT INTO testtable (id, data) VALUES (:id, :data)",
                 {'id':30, 'data':'d1'}
@@ -284,7 +286,7 @@ class InsertTest(AssertMixin):
             table.insert(inline=True).execute({'id':33, 'data':'d7'})
             table.insert(inline=True).execute({'data':'d8'})
 
-        self.assert_sql(testbase.db, go, [], with_sequences=[
+        self.assert_sql(testing.db, go, [], with_sequences=[
             (
                 "INSERT INTO testtable (id, data) VALUES (:id, :data)",
                 {'id':30, 'data':'d1'}
@@ -351,7 +353,7 @@ class InsertTest(AssertMixin):
 
         # test the same series of events using a reflected
         # version of the table
-        m2 = MetaData(testbase.db)
+        m2 = MetaData(testing.db)
         table = Table(table.name, m2, autoload=True)
         table.insert().execute({'id':30, 'data':'d1'})
         try:
@@ -375,13 +377,13 @@ class InsertTest(AssertMixin):
             (33, 'd4'),
         ]
 
-class DomainReflectionTest(AssertMixin):
+class DomainReflectionTest(TestBase, AssertsExecutionResults):
     "Test PostgreSQL domains"
 
     __only_on__ = 'postgres'
 
     def setUpAll(self):
-        con = testbase.db.connect()
+        con = testing.db.connect()
         try:
             con.execute('CREATE DOMAIN testdomain INTEGER NOT NULL DEFAULT 42')
             con.execute('CREATE DOMAIN alt_schema.testdomain INTEGER DEFAULT 0')
@@ -393,7 +395,7 @@ class DomainReflectionTest(AssertMixin):
         con.execute('CREATE TABLE crosschema (question integer, answer alt_schema.testdomain)')
 
     def tearDownAll(self):
-        con = testbase.db.connect()
+        con = testing.db.connect()
         con.execute('DROP TABLE testtable')
         con.execute('DROP TABLE alt_schema.testtable')
         con.execute('DROP TABLE crosschema')
@@ -401,47 +403,47 @@ class DomainReflectionTest(AssertMixin):
         con.execute('DROP DOMAIN alt_schema.testdomain')
 
     def test_table_is_reflected(self):
-        metadata = MetaData(testbase.db)
+        metadata = MetaData(testing.db)
         table = Table('testtable', metadata, autoload=True)
         self.assertEquals(set(table.columns.keys()), set(['question', 'answer']), "Columns of reflected table didn't equal expected columns")
         self.assertEquals(table.c.answer.type.__class__, postgres.PGInteger)
 
     def test_domain_is_reflected(self):
-        metadata = MetaData(testbase.db)
+        metadata = MetaData(testing.db)
         table = Table('testtable', metadata, autoload=True)
         self.assertEquals(str(table.columns.answer.default.arg), '42', "Reflected default value didn't equal expected value")
         self.assertFalse(table.columns.answer.nullable, "Expected reflected column to not be nullable.")
 
     def test_table_is_reflected_alt_schema(self):
-        metadata = MetaData(testbase.db)
+        metadata = MetaData(testing.db)
         table = Table('testtable', metadata, autoload=True, schema='alt_schema')
         self.assertEquals(set(table.columns.keys()), set(['question', 'answer', 'anything']), "Columns of reflected table didn't equal expected columns")
         self.assertEquals(table.c.anything.type.__class__, postgres.PGInteger)
 
     def test_schema_domain_is_reflected(self):
-        metadata = MetaData(testbase.db)
+        metadata = MetaData(testing.db)
         table = Table('testtable', metadata, autoload=True, schema='alt_schema')
         self.assertEquals(str(table.columns.answer.default.arg), '0', "Reflected default value didn't equal expected value")
         self.assertTrue(table.columns.answer.nullable, "Expected reflected column to be nullable.")
 
     def test_crosschema_domain_is_reflected(self):
-        metadata = MetaData(testbase.db)
+        metadata = MetaData(testing.db)
         table = Table('crosschema', metadata, autoload=True)
         self.assertEquals(str(table.columns.answer.default.arg), '0', "Reflected default value didn't equal expected value")
         self.assertTrue(table.columns.answer.nullable, "Expected reflected column to be nullable.")
 
-class MiscTest(AssertMixin):
+class MiscTest(TestBase, AssertsExecutionResults):
     __only_on__ = 'postgres'
 
     def test_date_reflection(self):
-        m1 = MetaData(testbase.db)
+        m1 = MetaData(testing.db)
         t1 = Table('pgdate', m1,
             Column('date1', DateTime(timezone=True)),
             Column('date2', DateTime(timezone=False))
             )
         m1.create_all()
         try:
-            m2 = MetaData(testbase.db)
+            m2 = MetaData(testing.db)
             t2 = Table('pgdate', m2, autoload=True)
             assert t2.c.date1.type.timezone is True
             assert t2.c.date2.type.timezone is False
@@ -449,7 +451,7 @@ class MiscTest(AssertMixin):
             m1.drop_all()
 
     def test_pg_weirdchar_reflection(self):
-        meta1 = MetaData(testbase.db)
+        meta1 = MetaData(testing.db)
         subject = Table("subject", meta1,
                         Column("id$", Integer, primary_key=True),
                         )
@@ -460,7 +462,7 @@ class MiscTest(AssertMixin):
                         )
         meta1.create_all()
         try:
-            meta2 = MetaData(testbase.db)
+            meta2 = MetaData(testing.db)
             subject = Table("subject", meta2, autoload=True)
             referer = Table("referer", meta2, autoload=True)
             print str(subject.join(referer).onclause)
@@ -469,17 +471,17 @@ class MiscTest(AssertMixin):
             meta1.drop_all()
 
     def test_checksfor_sequence(self):
-        meta1 = MetaData(testbase.db)
+        meta1 = MetaData(testing.db)
         t = Table('mytable', meta1,
             Column('col1', Integer, Sequence('fooseq')))
         try:
-            testbase.db.execute("CREATE SEQUENCE fooseq")
+            testing.db.execute("CREATE SEQUENCE fooseq")
             t.create(checkfirst=True)
         finally:
             t.drop(checkfirst=True)
 
     def test_distinct_on(self):
-        t = Table('mytable', MetaData(testbase.db),
+        t = Table('mytable', MetaData(testing.db),
                   Column('id', Integer, primary_key=True),
                   Column('a', String(8)))
         self.assertEquals(
@@ -498,7 +500,7 @@ class MiscTest(AssertMixin):
     def test_schema_reflection(self):
         """note: this test requires that the 'alt_schema' schema be separate and accessible by the test user"""
 
-        meta1 = MetaData(testbase.db)
+        meta1 = MetaData(testing.db)
         users = Table('users', meta1,
             Column('user_id', Integer, primary_key = True),
             Column('user_name', String(30), nullable = False),
@@ -513,7 +515,7 @@ class MiscTest(AssertMixin):
         )
         meta1.create_all()
         try:
-            meta2 = MetaData(testbase.db)
+            meta2 = MetaData(testing.db)
             addresses = Table('email_addresses', meta2, autoload=True, schema="alt_schema")
             users = Table('users', meta2, mustexist=True, schema="alt_schema")
 
@@ -526,7 +528,7 @@ class MiscTest(AssertMixin):
             meta1.drop_all()
 
     def test_schema_reflection_2(self):
-        meta1 = MetaData(testbase.db)
+        meta1 = MetaData(testing.db)
         subject = Table("subject", meta1,
                         Column("id", Integer, primary_key=True),
                         )
@@ -537,7 +539,7 @@ class MiscTest(AssertMixin):
                         schema="alt_schema")
         meta1.create_all()
         try:
-            meta2 = MetaData(testbase.db)
+            meta2 = MetaData(testing.db)
             subject = Table("subject", meta2, autoload=True)
             referer = Table("referer", meta2, schema="alt_schema", autoload=True)
             print str(subject.join(referer).onclause)
@@ -546,7 +548,7 @@ class MiscTest(AssertMixin):
             meta1.drop_all()
 
     def test_schema_reflection_3(self):
-        meta1 = MetaData(testbase.db)
+        meta1 = MetaData(testing.db)
         subject = Table("subject", meta1,
                         Column("id", Integer, primary_key=True),
                         schema='alt_schema_2'
@@ -559,22 +561,46 @@ class MiscTest(AssertMixin):
 
         meta1.create_all()
         try:
-            meta2 = MetaData(testbase.db)
+            meta2 = MetaData(testing.db)
             subject = Table("subject", meta2, autoload=True, schema="alt_schema_2")
             referer = Table("referer", meta2, schema="alt_schema", autoload=True)
             print str(subject.join(referer).onclause)
             self.assert_((subject.c.id==referer.c.ref).compare(subject.join(referer).onclause))
         finally:
             meta1.drop_all()
-
+    
+    def test_schema_roundtrips(self):
+        meta = MetaData(testing.db)
+        users = Table('users', meta, 
+            Column('id', Integer, primary_key=True),
+            Column('name', String(50)), schema='alt_schema')
+        users.create()
+        try:
+            users.insert().execute(id=1, name='name1')
+            users.insert().execute(id=2, name='name2')
+            users.insert().execute(id=3, name='name3')
+            users.insert().execute(id=4, name='name4')
+            
+            self.assertEquals(users.select().where(users.c.name=='name2').execute().fetchall(), [(2, 'name2')])
+            self.assertEquals(users.select(use_labels=True).where(users.c.name=='name2').execute().fetchall(), [(2, 'name2')])
+            
+            users.delete().where(users.c.id==3).execute()
+            self.assertEquals(users.select().where(users.c.name=='name3').execute().fetchall(), [])
+            
+            users.update().where(users.c.name=='name4').execute(name='newname')
+            self.assertEquals(users.select(use_labels=True).where(users.c.id==4).execute().fetchall(), [(4, 'newname')])
+            
+        finally:
+            users.drop()
+            
     def test_preexecute_passivedefault(self):
         """test that when we get a primary key column back
         from reflecting a table which has a default value on it, we pre-execute
         that PassiveDefault upon insert."""
 
         try:
-            meta = MetaData(testbase.db)
-            testbase.db.execute("""
+            meta = MetaData(testing.db)
+            testing.db.execute("""
              CREATE TABLE speedy_users
              (
                  speedy_user_id   SERIAL     PRIMARY KEY,
@@ -590,7 +616,7 @@ class MiscTest(AssertMixin):
             l = t.select().execute().fetchall()
             assert l == [(1, 'user', 'lala')]
         finally:
-            testbase.db.execute("drop table speedy_users", None)
+            testing.db.execute("drop table speedy_users", None)
 
     def test_create_partial_index(self):
         tbl = Table('testtbl', MetaData(), Column('data',Integer))
@@ -604,7 +630,7 @@ class MiscTest(AssertMixin):
 
         assert executed_sql == ['CREATE INDEX test_idx1 ON testtbl (data) WHERE testtbl.data > 5 AND testtbl.data < 10']
 
-class TimezoneTest(AssertMixin):
+class TimezoneTest(TestBase, AssertsExecutionResults):
     """Test timezone-aware datetimes.
 
     psycopg will return a datetime with a tzinfo attached to it, if postgres
@@ -617,7 +643,7 @@ class TimezoneTest(AssertMixin):
 
     def setUpAll(self):
         global tztable, notztable, metadata
-        metadata = MetaData(testbase.db)
+        metadata = MetaData(testing.db)
 
         # current_timestamp() in postgres is assumed to return TIMESTAMP WITH TIMEZONE
         tztable = Table('tztable', metadata,
@@ -636,7 +662,7 @@ class TimezoneTest(AssertMixin):
 
     def test_with_timezone(self):
         # get a date with a tzinfo
-        somedate = testbase.db.connect().scalar(func.current_timestamp().select())
+        somedate = testing.db.connect().scalar(func.current_timestamp().select())
         tztable.insert().execute(id=1, name='row1', date=somedate)
         c = tztable.update(tztable.c.id==1).execute(name='newname')
         print tztable.select(tztable.c.id==1).execute().fetchone()
@@ -648,12 +674,12 @@ class TimezoneTest(AssertMixin):
         c = notztable.update(notztable.c.id==1).execute(name='newname')
         print notztable.select(tztable.c.id==1).execute().fetchone()
 
-class ArrayTest(AssertMixin):
+class ArrayTest(TestBase, AssertsExecutionResults):
     __only_on__ = 'postgres'
 
     def setUpAll(self):
         global metadata, arrtable
-        metadata = MetaData(testbase.db)
+        metadata = MetaData(testing.db)
 
         arrtable = Table('arrtable', metadata,
             Column('id', Integer, primary_key=True),
@@ -665,7 +691,7 @@ class ArrayTest(AssertMixin):
         metadata.drop_all()
 
     def test_reflect_array_column(self):
-        metadata2 = MetaData(testbase.db)
+        metadata2 = MetaData(testing.db)
         tbl = Table('arrtable', metadata2, autoload=True)
         self.assertTrue(isinstance(tbl.c.intarr.type, postgres.PGArray))
         self.assertTrue(isinstance(tbl.c.strarr.type, postgres.PGArray))
@@ -704,5 +730,87 @@ class ArrayTest(AssertMixin):
         self.assertEquals(results[1]['strarr'], [[u'm\xe4\xe4'], [u'm\xf6\xf6']])
         arrtable.delete().execute()
 
+    def test_array_mutability(self):
+        class Foo(object): pass
+        footable = Table('foo', metadata,
+            Column('id', Integer, primary_key=True),
+            Column('intarr', postgres.PGArray(Integer), nullable=True)
+        )
+        mapper(Foo, footable)
+        metadata.create_all()
+        sess = create_session()
+        
+        foo = Foo()
+        foo.id = 1
+        foo.intarr = [1,2,3]
+        sess.save(foo)
+        sess.flush()
+        sess.clear()
+        foo = sess.query(Foo).get(1)
+        self.assertEquals(foo.intarr, [1,2,3])
+        
+        foo.intarr.append(4)
+        sess.flush()
+        sess.clear()
+        foo = sess.query(Foo).get(1)
+        self.assertEquals(foo.intarr, [1,2,3,4])
+        
+        foo.intarr = []
+        sess.flush()
+        sess.clear()
+        self.assertEquals(foo.intarr, [])
+        
+        foo.intarr = None
+        sess.flush()
+        sess.clear()
+        self.assertEquals(foo.intarr, None)
+
+        # Errors in r4217:
+        foo = Foo()
+        foo.id = 2
+        sess.save(foo)
+        sess.flush()
+
+class TimeStampTest(TestBase, AssertsExecutionResults): 
+    __only_on__ = 'postgres' 
+    def test_timestamp(self): 
+        engine = testing.db
+        connection = engine.connect() 
+        s = select([func.TIMESTAMP("12/25/07").label("ts")]) 
+        result = connection.execute(s).fetchone() 
+        self.assertEqual(result[0], datetime.datetime(2007, 12, 25, 0, 0)) 
+
+class ServerSideCursorsTest(TestBase, AssertsExecutionResults):
+    __only_on__ = 'postgres'
+    
+    def setUpAll(self):
+        global ss_engine
+        ss_engine = engines.testing_engine(options={'server_side_cursors':True})
+        
+    def tearDownAll(self):
+        ss_engine.dispose()
+    
+    def test_roundtrip(self):
+        test_table = Table('test_table', MetaData(ss_engine),
+            Column('id', Integer, primary_key=True),
+            Column('data', String(50))
+        )
+        test_table.create(checkfirst=True)
+        try:
+            test_table.insert().execute(data='data1')
+            
+            nextid = ss_engine.execute(Sequence('test_table_id_seq'))
+            test_table.insert().execute(id=nextid, data='data2')
+            
+            self.assertEquals(test_table.select().execute().fetchall(), [(1, 'data1'), (2, 'data2')])
+            
+            test_table.update().where(test_table.c.id==2).values(data=test_table.c.data + ' updated').execute()
+            self.assertEquals(test_table.select().execute().fetchall(), [(1, 'data1'), (2, 'data2 updated')])
+            test_table.delete().execute()
+            self.assertEquals(test_table.count().scalar(), 0)
+        finally:
+            test_table.drop(checkfirst=True)
+            
+    
 if __name__ == "__main__":
-    testbase.main()
+    testenv.main()
