@@ -1,8 +1,9 @@
-import testbase
+import testenv; testenv.configure_for_tests()
 import gc
 from sqlalchemy import MetaData, Integer, String, ForeignKey
 from sqlalchemy.orm import mapper, relation, clear_mappers, create_session
-from sqlalchemy.orm.mapper import Mapper
+from sqlalchemy.orm.mapper import Mapper, _mapper_registry
+from sqlalchemy.orm.session import _sessions 
 from testlib import *
 from testlib.fixtures import Base
 
@@ -19,31 +20,47 @@ def profile_memory(func):
             gc.collect()
             samples.append(len(gc.get_objects()))
         print "sample gc sizes:", samples
-        # TODO: this test only finds pure "growing" tests
+
+        assert len(_sessions) == 0
+
+        # TODO: this test only finds pure "growing" tests.
+        # if a drop is detected, it's assumed that GC is able
+        # to reduce memory.  better methodology would
+        # make this more accurate.
         for i, x in enumerate(samples):
-            if i < len(samples) - 1 and samples[i+1] <= x:
-                break
+            if i < len(samples) - 1 and x < samples[i+1]:
+                continue
         else:
-            assert False
-        assert True
+            return
+        assert False, repr(samples)
     return profile
 
-class MemUsageTest(AssertMixin):
-    
-    def test_session(self):
-        metadata = MetaData(testbase.db)
+def assert_no_mappers():
+    clear_mappers()
+    gc.collect()
+    assert len(_mapper_registry) == 0
 
-        table1 = Table("mytable", metadata, 
+class EnsureZeroed(TestBase, AssertsExecutionResults):
+    def setUp(self):
+        _sessions.clear()
+        _mapper_registry.clear()
+        
+class MemUsageTest(EnsureZeroed):
+
+    def test_session(self):
+        metadata = MetaData(testing.db)
+
+        table1 = Table("mytable", metadata,
             Column('col1', Integer, primary_key=True),
             Column('col2', String(30))
             )
 
-        table2 = Table("mytable2", metadata, 
+        table2 = Table("mytable2", metadata,
             Column('col1', Integer, primary_key=True),
             Column('col2', String(30)),
             Column('col3', Integer, ForeignKey("mytable.col1"))
             )
-    
+
         metadata.create_all()
 
         m1 = mapper(A, table1, properties={
@@ -52,7 +69,7 @@ class MemUsageTest(AssertMixin):
         m2 = mapper(B, table2)
 
         m3 = mapper(A, table1, non_primary=True)
-        
+
         @profile_memory
         def go():
             sess = create_session()
@@ -70,29 +87,30 @@ class MemUsageTest(AssertMixin):
             alist = sess.query(A).all()
             self.assertEquals(
                 [
-                    A(col2="a1", bs=[B(col2="b1"), B(col2="b2")]), 
-                    A(col2="a2", bs=[]), 
+                    A(col2="a1", bs=[B(col2="b1"), B(col2="b2")]),
+                    A(col2="a2", bs=[]),
                     A(col2="a3", bs=[B(col2="b3")])
-                ], 
+                ],
                 alist)
-            
+
             for a in alist:
                 sess.delete(a)
             sess.flush()
         go()
-        
-        metadata.drop_all()
-        clear_mappers()
-        
-    def test_mapper_reset(self):
-        metadata = MetaData(testbase.db)
 
-        table1 = Table("mytable", metadata, 
+        metadata.drop_all()
+        del m1, m2, m3
+        assert_no_mappers()
+
+    def test_mapper_reset(self):
+        metadata = MetaData(testing.db)
+
+        table1 = Table("mytable", metadata,
             Column('col1', Integer, primary_key=True),
             Column('col2', String(30))
             )
 
-        table2 = Table("mytable2", metadata, 
+        table2 = Table("mytable2", metadata,
             Column('col1', Integer, primary_key=True),
             Column('col2', String(30)),
             Column('col3', Integer, ForeignKey("mytable.col1"))
@@ -106,7 +124,7 @@ class MemUsageTest(AssertMixin):
             m2 = mapper(B, table2)
 
             m3 = mapper(A, table1, non_primary=True)
-        
+
             sess = create_session()
             a1 = A(col2="a1")
             a2 = A(col2="a2")
@@ -122,32 +140,34 @@ class MemUsageTest(AssertMixin):
             alist = sess.query(A).all()
             self.assertEquals(
                 [
-                    A(col2="a1", bs=[B(col2="b1"), B(col2="b2")]), 
-                    A(col2="a2", bs=[]), 
+                    A(col2="a1", bs=[B(col2="b1"), B(col2="b2")]),
+                    A(col2="a2", bs=[]),
                     A(col2="a3", bs=[B(col2="b3")])
-                ], 
+                ],
                 alist)
-        
+
             for a in alist:
                 sess.delete(a)
             sess.flush()
+            sess.close()
             clear_mappers()
-        
+
         metadata.create_all()
         try:
             go()
         finally:
             metadata.drop_all()
+        assert_no_mappers()
 
     def test_with_inheritance(self):
-        metadata = MetaData(testbase.db)
+        metadata = MetaData(testing.db)
 
-        table1 = Table("mytable", metadata, 
+        table1 = Table("mytable", metadata,
             Column('col1', Integer, primary_key=True),
             Column('col2', String(30))
             )
 
-        table2 = Table("mytable2", metadata, 
+        table2 = Table("mytable2", metadata,
             Column('col1', Integer, ForeignKey('mytable.col1'), primary_key=True),
             Column('col3', String(30)),
             )
@@ -158,10 +178,10 @@ class MemUsageTest(AssertMixin):
                 pass
             class B(A):
                 pass
-            
+
             mapper(A, table1, polymorphic_on=table1.c.col2, polymorphic_identity='a')
             mapper(B, table2, inherits=A, polymorphic_identity='b')
-            
+
             sess = create_session()
             a1 = A()
             a2 = A()
@@ -176,13 +196,13 @@ class MemUsageTest(AssertMixin):
             self.assertEquals(
                 [
                     A(), A(), B(col3='b1'), B(col3='b2')
-                ], 
+                ],
                 alist)
 
             for a in alist:
                 sess.delete(a)
             sess.flush()
-            
+
             # dont need to clear_mappers()
             del B
             del A
@@ -192,20 +212,21 @@ class MemUsageTest(AssertMixin):
             go()
         finally:
             metadata.drop_all()
+        assert_no_mappers()
 
     def test_with_manytomany(self):
-        metadata = MetaData(testbase.db)
+        metadata = MetaData(testing.db)
 
-        table1 = Table("mytable", metadata, 
+        table1 = Table("mytable", metadata,
             Column('col1', Integer, primary_key=True),
             Column('col2', String(30))
             )
 
-        table2 = Table("mytable2", metadata, 
+        table2 = Table("mytable2", metadata,
             Column('col1', Integer, primary_key=True),
             Column('col2', String(30)),
             )
-        
+
         table3 = Table('t1tot2', metadata,
             Column('t1', Integer, ForeignKey('mytable.col1')),
             Column('t2', Integer, ForeignKey('mytable2.col1')),
@@ -239,7 +260,7 @@ class MemUsageTest(AssertMixin):
             self.assertEquals(
                 [
                     A(bs=[B(col2='b1')]), A(bs=[B(col2='b2')])
-                ], 
+                ],
                 alist)
 
             for a in alist:
@@ -255,7 +276,8 @@ class MemUsageTest(AssertMixin):
             go()
         finally:
             metadata.drop_all()
+        assert_no_mappers()
+
 
-    
 if __name__ == '__main__':
-    testbase.main()
+    testenv.main()
