@@ -1,6 +1,6 @@
-import testbase
+import testenv; testenv.configure_for_tests()
 from sqlalchemy import *
-from sqlalchemy import exceptions
+from sqlalchemy import exceptions, util
 from sqlalchemy.orm import *
 from sqlalchemy.orm.session import SessionExtension
 from sqlalchemy.orm.session import Session as SessionCls
@@ -11,7 +11,7 @@ import pickle
 import gc
 
 
-class SessionTest(AssertMixin):
+class SessionTest(TestBase, AssertsExecutionResults):
     def setUpAll(self):
         tables.create()
 
@@ -28,7 +28,7 @@ class SessionTest(AssertMixin):
 
     def test_close(self):
         """test that flush() doenst close a connection the session didnt open"""
-        c = testbase.db.connect()
+        c = testing.db.connect()
         class User(object):pass
         mapper(User, users)
         s = create_session(bind=c)
@@ -45,7 +45,7 @@ class SessionTest(AssertMixin):
         s.flush()
 
     def test_close_two(self):
-        c = testbase.db.connect()
+        c = testing.db.connect()
         try:
             class User(object):pass
             mapper(User, users)
@@ -86,7 +86,7 @@ class SessionTest(AssertMixin):
     @engines.close_open_connections
     def test_binds_from_expression(self):
         """test that Session can extract Table objects from ClauseElements and match them to tables."""
-        Session = sessionmaker(binds={users:testbase.db, addresses:testbase.db})
+        Session = sessionmaker(binds={users:testing.db, addresses:testing.db})
         sess = Session()
         sess.execute(users.insert(), params=dict(user_id=1, user_name='ed'))
         assert sess.execute(users.select()).fetchall() == [(1, 'ed')]
@@ -95,18 +95,33 @@ class SessionTest(AssertMixin):
         mapper(User, users, properties={
             'addresses':relation(Address, backref=backref("user", cascade="all"), cascade="all")
         })
-        Session = sessionmaker(binds={User:testbase.db, Address:testbase.db})
+        Session = sessionmaker(binds={User:testing.db, Address:testing.db})
         sess.execute(users.insert(), params=dict(user_id=2, user_name='fred'))
         assert sess.execute(users.select()).fetchall() == [(1, 'ed'), (2, 'fred')]
         sess.close()
 
+    @engines.close_open_connections
+    def test_bind_from_metadata(self):
+        Session = sessionmaker()
+        sess = Session()
+        mapper(User, users)
+
+        sess.execute(users.insert(), dict(user_name='Johnny'))
+
+        assert len(sess.query(User).all()) == 1
+
+        sess.execute(users.delete())
+
+        assert len(sess.query(User).all()) == 0
+        sess.close()
+
     @testing.unsupported('sqlite', 'mssql') # TEMP: test causes mssql to hang
     @engines.close_open_connections
     def test_transaction(self):
         class User(object):pass
         mapper(User, users)
-        conn1 = testbase.db.connect()
-        conn2 = testbase.db.connect()
+        conn1 = testing.db.connect()
+        conn2 = testing.db.connect()
 
         sess = create_session(transactional=True, bind=conn1)
         u = User()
@@ -116,16 +131,30 @@ class SessionTest(AssertMixin):
         assert conn2.execute("select count(1) from users").scalar() == 0
         sess.commit()
         assert conn1.execute("select count(1) from users").scalar() == 1
-        assert testbase.db.connect().execute("select count(1) from users").scalar() == 1
+        assert testing.db.connect().execute("select count(1) from users").scalar() == 1
         sess.close()
 
+    def test_flush_noop(self):
+        session = create_session()
+        session.uow = object()
+
+        self.assertRaises(AttributeError, session.flush)
+
+        session = create_session()
+        session.uow = object()
+
+        session.flush(objects=[])
+        session.flush(objects=set())
+        session.flush(objects=())
+        session.flush(objects=iter([]))
+
     @testing.unsupported('sqlite', 'mssql') # TEMP: test causes mssql to hang
     @engines.close_open_connections
     def test_autoflush(self):
         class User(object):pass
         mapper(User, users)
-        conn1 = testbase.db.connect()
-        conn2 = testbase.db.connect()
+        conn1 = testing.db.connect()
+        conn2 = testing.db.connect()
 
         sess = create_session(bind=conn1, transactional=True, autoflush=True)
         u = User()
@@ -137,9 +166,24 @@ class SessionTest(AssertMixin):
         assert conn2.execute("select count(1) from users").scalar() == 0
         sess.commit()
         assert conn1.execute("select count(1) from users").scalar() == 1
-        assert testbase.db.connect().execute("select count(1) from users").scalar() == 1
+        assert testing.db.connect().execute("select count(1) from users").scalar() == 1
         sess.close()
 
+    def test_autoflush_expressions(self):
+        class User(fixtures.Base):
+            pass
+        class Address(fixtures.Base):
+            pass
+        mapper(User, users, properties={
+            'addresses':relation(Address, backref="user")
+        })
+        mapper(Address, addresses)
+
+        sess = create_session(autoflush=True, transactional=True)
+        u = User(user_name='ed', addresses=[Address(email_address='foo')])
+        sess.save(u)
+        self.assertEquals(sess.query(Address).filter(Address.user==u).one(), Address(email_address='foo'))
+
     @testing.unsupported('sqlite', 'mssql') # TEMP: test causes mssql to hang
     @engines.close_open_connections
     def test_autoflush_unbound(self):
@@ -154,10 +198,10 @@ class SessionTest(AssertMixin):
             u2 = sess.query(User).filter_by(user_name='ed').one()
             assert u2 is u
             assert sess.execute("select count(1) from users", mapper=User).scalar() == 1
-            assert testbase.db.connect().execute("select count(1) from users").scalar() == 0
+            assert testing.db.connect().execute("select count(1) from users").scalar() == 0
             sess.commit()
             assert sess.execute("select count(1) from users", mapper=User).scalar() == 1
-            assert testbase.db.connect().execute("select count(1) from users").scalar() == 1
+            assert testing.db.connect().execute("select count(1) from users").scalar() == 1
             sess.close()
         except:
             sess.rollback()
@@ -167,8 +211,8 @@ class SessionTest(AssertMixin):
     def test_autoflush_2(self):
         class User(object):pass
         mapper(User, users)
-        conn1 = testbase.db.connect()
-        conn2 = testbase.db.connect()
+        conn1 = testing.db.connect()
+        conn2 = testing.db.connect()
 
         sess = create_session(bind=conn1, transactional=True, autoflush=True)
         u = User()
@@ -176,7 +220,7 @@ class SessionTest(AssertMixin):
         sess.save(u)
         sess.commit()
         assert conn1.execute("select count(1) from users").scalar() == 1
-        assert testbase.db.connect().execute("select count(1) from users").scalar() == 1
+        assert testing.db.connect().execute("select count(1) from users").scalar() == 1
         sess.commit()
 
     # TODO: not doing rollback of attributes right now.
@@ -206,7 +250,7 @@ class SessionTest(AssertMixin):
     def test_external_joined_transaction(self):
         class User(object):pass
         mapper(User, users)
-        conn = testbase.db.connect()
+        conn = testing.db.connect()
         trans = conn.begin()
         sess = create_session(bind=conn, transactional=True, autoflush=True)
         sess.begin()
@@ -215,7 +259,7 @@ class SessionTest(AssertMixin):
         sess.flush()
         sess.commit() # commit does nothing
         trans.rollback() # rolls back
-        assert len(sess.query(User).select()) == 0
+        assert len(sess.query(User).all()) == 0
         sess.close()
 
     @testing.unsupported('sqlite', 'mssql', 'firebird', 'sybase', 'access',
@@ -225,7 +269,7 @@ class SessionTest(AssertMixin):
         class User(object):pass
         mapper(User, users)
         try:
-            conn = testbase.db.connect()
+            conn = testing.db.connect()
             trans = conn.begin()
             sess = create_session(bind=conn, transactional=True, autoflush=True)
             u1 = User()
@@ -239,7 +283,7 @@ class SessionTest(AssertMixin):
             sess.rollback()
 
             trans.commit()
-            assert len(sess.query(User).select()) == 1
+            assert len(sess.query(User).all()) == 1
         except:
             conn.close()
             raise
@@ -248,7 +292,7 @@ class SessionTest(AssertMixin):
                          'oracle', 'maxdb')
     @engines.close_open_connections
     def test_heavy_nesting(self):
-        session = create_session(bind=testbase.db)
+        session = create_session(bind=testing.db)
 
         session.begin()
         session.connection().execute("insert into users (user_name) values ('user1')")
@@ -279,9 +323,9 @@ class SessionTest(AssertMixin):
         mapper(User, users)
         mapper(Address, addresses)
 
-        engine2 = create_engine(testbase.db.url)
+        engine2 = create_engine(testing.db.url)
         sess = create_session(transactional=False, autoflush=False, twophase=True)
-        sess.bind_mapper(User, testbase.db)
+        sess.bind_mapper(User, testing.db)
         sess.bind_mapper(Address, engine2)
         sess.begin()
         u1 = User()
@@ -304,7 +348,7 @@ class SessionTest(AssertMixin):
         sess.flush()
         sess.commit() # commit does nothing
         sess.rollback() # rolls back
-        assert len(sess.query(User).select()) == 0
+        assert len(sess.query(User).all()) == 0
         sess.close()
 
     @testing.unsupported('sqlite', 'mssql', 'firebird', 'sybase', 'access',
@@ -329,7 +373,7 @@ class SessionTest(AssertMixin):
         sess.rollback()
 
         sess.commit()
-        assert len(sess.query(User).select()) == 1
+        assert len(sess.query(User).all()) == 1
         sess.close()
 
     @testing.unsupported('sqlite', 'mssql', 'firebird', 'sybase', 'access',
@@ -352,48 +396,164 @@ class SessionTest(AssertMixin):
         sess.rollback()
 
         sess.commit()
-        assert len(sess.query(User).select()) == 1
+        assert len(sess.query(User).all()) == 1
+        sess.close()
+
+    @testing.unsupported('sqlite', 'mssql', 'firebird', 'sybase', 'access',
+                         'oracle', 'maxdb')
+    @testing.exclude('mysql', '<', (5, 0, 3))
+    def test_nested_transaction_connection_add(self):
+        class User(object): pass
+        mapper(User, users)
+
+        sess = create_session(transactional=False)
+
+        sess.begin()
+        sess.begin_nested()
+
+        u1 = User()
+        sess.save(u1)
+        sess.flush()
+
+        sess.rollback()
+
+        u2 = User()
+        sess.save(u2)
+
+        sess.commit()
+
+        self.assertEquals(util.Set(sess.query(User).all()), util.Set([u2]))
+
+        sess.begin()
+        sess.begin_nested()
+
+        u3 = User()
+        sess.save(u3)
+        sess.commit() # commit the nested transaction
+        sess.rollback()
+
+        self.assertEquals(util.Set(sess.query(User).all()), util.Set([u2]))
+
+        sess.close()
+
+    @testing.unsupported('sqlite', 'mssql', 'firebird', 'sybase', 'access',
+                         'oracle', 'maxdb')
+    @testing.exclude('mysql', '<', (5, 0, 3))
+    def test_mixed_transaction_control(self):
+        class User(object): pass
+        mapper(User, users)
+
+        sess = create_session(transactional=False)
+
+        sess.begin()
+        sess.begin_nested()
+        transaction = sess.begin()
+
+        sess.save(User())
+
+        transaction.commit()
+        sess.commit()
+        sess.commit()
+
+        sess.close()
+
+        self.assertEquals(len(sess.query(User).all()), 1)
+
+        t1 = sess.begin()
+        t2 = sess.begin_nested()
+
+        sess.save(User())
+
+        t2.commit()
+        assert sess.transaction is t1
+
+        sess.close()
+
+    @testing.unsupported('sqlite', 'mssql', 'firebird', 'sybase', 'access',
+                         'oracle', 'maxdb')
+    @testing.exclude('mysql', '<', (5, 0, 3))
+    def test_mixed_transaction_close(self):
+        class User(object): pass
+        mapper(User, users)
+
+        sess = create_session(transactional=True)
+
+        sess.begin_nested()
+
+        sess.save(User())
+        sess.flush()
+
+        sess.close()
+
+        sess.save(User())
+        sess.commit()
+
+        sess.close()
+
+        self.assertEquals(len(sess.query(User).all()), 1)
+
+    @testing.unsupported('sqlite', 'mssql', 'firebird', 'sybase', 'access',
+                         'oracle', 'maxdb')
+    @testing.exclude('mysql', '<', (5, 0, 3))
+    def test_error_on_using_inactive_session(self):
+        class User(object): pass
+        mapper(User, users)
+
+        sess = create_session(transactional=False)
+
+        try:
+            sess.begin()
+            sess.begin()
+
+            sess.save(User())
+            sess.flush()
+
+            sess.rollback()
+            sess.begin()
+            assert False
+        except exceptions.InvalidRequestError, e:
+            self.assertEquals(str(e), "The transaction is inactive due to a rollback in a subtransaction and should be closed")
         sess.close()
 
     @engines.close_open_connections
     def test_bound_connection(self):
         class User(object):pass
         mapper(User, users)
-        c = testbase.db.connect()
+        c = testing.db.connect()
         sess = create_session(bind=c)
         sess.create_transaction()
         transaction = sess.transaction
         u = User()
         sess.save(u)
         sess.flush()
-        assert transaction.get_or_add(testbase.db) is transaction.get_or_add(c) is c
+        assert transaction.get_or_add(testing.db) is transaction.get_or_add(c) is c
 
         try:
-            transaction.add(testbase.db.connect())
+            transaction.add(testing.db.connect())
             assert False
         except exceptions.InvalidRequestError, e:
             assert str(e) == "Session already has a Connection associated for the given Connection's Engine"
 
         try:
-            transaction.get_or_add(testbase.db.connect())
+            transaction.get_or_add(testing.db.connect())
             assert False
         except exceptions.InvalidRequestError, e:
             assert str(e) == "Session already has a Connection associated for the given Connection's Engine"
 
         try:
-            transaction.add(testbase.db)
+            transaction.add(testing.db)
             assert False
         except exceptions.InvalidRequestError, e:
             assert str(e) == "Session already has a Connection associated for the given Engine"
 
         transaction.rollback()
-        assert len(sess.query(User).select()) == 0
+        assert len(sess.query(User).all()) == 0
         sess.close()
 
     def test_bound_connection_transactional(self):
         class User(object):pass
         mapper(User, users)
-        c = testbase.db.connect()
+        c = testing.db.connect()
 
         sess = create_session(bind=c, transactional=True)
         u = User()
@@ -413,7 +573,7 @@ class SessionTest(AssertMixin):
         c.execute("delete from users")
         assert c.scalar("select count(1) from users") == 0
 
-        c = testbase.db.connect()
+        c = testing.db.connect()
 
         trans = c.begin()
         sess = create_session(bind=c, transactional=False)
@@ -721,19 +881,20 @@ class SessionTest(AssertMixin):
                 log.append('after_flush')
             def after_flush_postexec(self, session, flush_context):
                 log.append('after_flush_postexec')
+            def after_begin(self, session, transaction, connection):
+                log.append('after_begin')
         sess = create_session(extension = MyExt())
         u = User()
         sess.save(u)
         sess.flush()
-
-        assert log == ['before_flush', 'after_flush', 'before_commit', 'after_commit', 'after_flush_postexec']
+        assert log == ['before_flush', 'after_begin', 'after_flush', 'before_commit', 'after_commit', 'after_flush_postexec']
 
         log = []
         sess = create_session(transactional=True, extension=MyExt())
         u = User()
         sess.save(u)
         sess.flush()
-        assert log == ['before_flush', 'after_flush', 'after_flush_postexec']
+        assert log == ['before_flush', 'after_begin', 'after_flush', 'after_flush_postexec']
 
         log = []
         u.user_name = 'ed'
@@ -743,6 +904,11 @@ class SessionTest(AssertMixin):
         log = []
         sess.commit()
         assert log == ['before_commit', 'after_commit']
+        
+        log = []
+        sess = create_session(transactional=True, extension=MyExt(), bind=testing.db)
+        conn = sess.connection()
+        assert log == ['after_begin']
 
     def test_pickled_update(self):
         mapper(User, users)
@@ -813,8 +979,8 @@ class SessionTest(AssertMixin):
         b = Bar()
         assert b in sess
         assert len(list(sess)) == 1
-        
-        
+
+
 class ScopedSessionTest(ORMTest):
 
     def define_tables(self, metadata):
@@ -830,8 +996,10 @@ class ScopedSessionTest(ORMTest):
     def test_basic(self):
         Session = scoped_session(sessionmaker())
 
-        class SomeObject(fixtures.Base):pass
-        class SomeOtherObject(fixtures.Base):pass
+        class SomeObject(fixtures.Base):
+            query = Session.query_property()
+        class SomeOtherObject(fixtures.Base):
+            query = Session.query_property()
 
         mapper(SomeObject, table, properties={
             'options':relation(SomeOtherObject)
@@ -845,16 +1013,17 @@ class ScopedSessionTest(ORMTest):
         Session.commit()
         Session.remove()
 
-        assert SomeObject(id=1, data="hello", options=[SomeOtherObject(someid=1)]) == Session.query(SomeObject).one()
+        self.assertEquals(SomeObject(id=1, data="hello", options=[SomeOtherObject(someid=1)]), Session.query(SomeObject).one())
+        self.assertEquals(SomeObject(id=1, data="hello", options=[SomeOtherObject(someid=1)]), SomeObject.query.one())
+        self.assertEquals(SomeOtherObject(someid=1), SomeOtherObject.query.filter(SomeOtherObject.someid==sso.someid).one())
 
-
-class ScopedMapperTest(PersistTest):
+class ScopedMapperTest(TestBase):
     def setUpAll(self):
         global metadata, table, table2
-        metadata = MetaData(testbase.db)
+        metadata = MetaData(testing.db)
         table = Table('sometable', metadata,
             Column('id', Integer, primary_key=True),
-            Column('data', String(30)))
+            Column('data', String(30), nullable=False))
         table2 = Table('someothertable', metadata,
             Column('id', Integer, primary_key=True),
             Column('someid', None, ForeignKey('sometable.id'))
@@ -935,6 +1104,36 @@ class ScopedMapperTest(PersistTest):
 
         assert MyClass().expunge() == "an expunge !"
 
+    def _test_autoflush_saveoninit(self, on_init, autoflush=None):
+        Session = scoped_session(
+            sessionmaker(transactional=True, autoflush=True))
+
+        class Foo(object):
+            def __init__(self, data=None):
+                if autoflush is not None:
+                    friends = Session.query(Foo).autoflush(autoflush).all()
+                else:
+                    friends = Session.query(Foo).all()
+                self.data = data
+
+        Session.mapper(Foo, table, save_on_init=on_init)
+
+        a1 = Foo('an address')
+        Session.flush()
+
+    def test_autoflush_saveoninit(self):
+        """Test save_on_init + query.autoflush()"""
+        self._test_autoflush_saveoninit(False)
+        self._test_autoflush_saveoninit(False, True)
+        self._test_autoflush_saveoninit(False, False)
+
+        self.assertRaises(exceptions.DBAPIError,
+                          self._test_autoflush_saveoninit, True)
+        self.assertRaises(exceptions.DBAPIError,
+                          self._test_autoflush_saveoninit, True, True)
+        self._test_autoflush_saveoninit(True, False)
+
+
 class ScopedMapperTest2(ORMTest):
     def define_tables(self, metadata):
         global table, table2
@@ -976,4 +1175,4 @@ class ScopedMapperTest2(ORMTest):
 
 
 if __name__ == "__main__":
-    testbase.main()
+    testenv.main()
